/* Talika panel applet: Switch between open windows using icons.
 * (C) 2009-2010 Sinew Software Systems
	 *
 * Authors:  Vinod Kumar Dahiya
 *           Hemant Kumar
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
	 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <gtk/gtk.h>
#include <libwnck/libwnck.h>
#include <math.h>

#include "tasklist.h"
#include "task.h"
#include "xutils.h"
#include "tooltips.h"



static void 
talika_tooltips_hide_popup (TalikaTaskList * tasklist);

static gboolean 
talika_tooltips_leave_notify_timeout (gpointer data);

static gboolean
talika_tooltips_popup_enter_notify (GtkWidget * widget,
                                    GdkEventCrossing * event,
                                    TalikaTaskList * tasklist);

/*############################ Preview widget ################################*/

static gboolean
talika_preview_button_press_event (GtkWidget * widget,
                                GdkEventButton * event, gpointer data)
{
	TalikaTask *task = data;
	
	if (event->button == 1)
	{
		/* is_most_recently_activated == is_active for click &
		 * sloppy focus methods.  We use the former here because
		 * 'mouse' focus provides a special case.  In that case, no
		 * window will be active, but if a window was the most
		 * recently active one (i.e. user moves mouse straight from
		                        * window to tasklist), then we should still minimize it.
									*/
		if (wnck_window_is_most_recently_activated
		    (task->window))
			task->was_active = TRUE;
		else
			task->was_active = FALSE;

		/* This is an "activate" callback function so gtk_get_current_event_time()
		* will suffice.
		 */
		if(task->tasklist->previews_hide_activation)
		{
			talika_tasklist_activate_task_window (task,
		                                      gtk_get_current_event_time (),TRUE);
		    talika_tooltips_hide_popup_immediatly (task);
		}
		else
		{
			talika_tasklist_activate_task_window (task,
		                                      gtk_get_current_event_time (),FALSE);
		}
			
	}
	else if (event->button == 2)
	{
		wnck_window_close (task->window,
		                   gtk_get_current_event_time ());
		talika_tooltips_hide_popup_immediatly (task);
	}
	else if (event->button == 3)
	{
		talika_tooltips_hide_popup_immediatly (task);
	}
	
	return FALSE;
}

static GtkWidget*
talika_preview_widget_for_window (WnckWindow *window, TalikaTask *task)
{
	GdkPixbuf *buf;
	guint size = task->tasklist->preview_size;

	//update cache first
	talika_preview_image_cache_update_window (task->tasklist->preview_cache,
	                                          window,
	                                          task->tasklist->preview_size,
	                                          task->tasklist->box);


	buf = 
		talika_preview_image_cache_get_image(task->tasklist->preview_cache,
		                                     window,&size);
	if(!buf)
	{
		GdkPixbuf *newbuf = talika_get_icon_from_xserver (window,
		                                    size/2);
		if(!newbuf)
			newbuf = gdk_pixbuf_copy (wnck_window_get_icon
				                          (window));
		
		buf = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
                            TRUE,
                            gdk_pixbuf_get_bits_per_sample (newbuf),
                            ( size * 5 )/3,
                            size);

		gdk_pixbuf_fill (buf,0xfdfdfdff);
		gdk_pixbuf_composite (newbuf,buf,(size*7)/12,size/4,
		                      size/2,size/2,
		                      (size*7)/12,size/4
		                      ,1,1,
		                      GDK_INTERP_BILINEAR,255);
		g_object_unref (newbuf);

		size = ( size * 5 )/3;
	}

	
	GtkWidget *image = gtk_image_new_from_pixbuf (buf);
	g_object_unref (buf);
	
	GtkWidget *icon = gtk_image_new_from_pixbuf(
	                                            wnck_window_get_mini_icon 
	                                            (window));

	const gchar *text = wnck_window_get_name (window);
	GtkWidget *label = gtk_label_new (text);
	//g_free (text);
	gtk_label_set_ellipsize (GTK_LABEL (label),
	                         PANGO_ELLIPSIZE_END);
	gtk_widget_set_size_request (label,size-26,20);
	_make_gtk_label_bold (GTK_LABEL (label));
	gtk_label_set_justify (GTK_LABEL(label),GTK_JUSTIFY_LEFT);

	GtkWidget *empty = gtk_label_new ("");
	GtkWidget *empty2 = gtk_label_new ("");
	gtk_widget_set_size_request (empty2,size,2);
	
	/*GtkWidget *close_button = gtk_button_new();
	GtkImage *close_image = gtk_image_new_from_stock(GTK_STOCK_CLOSE);
	gtk_container_add(GTK_CONTAINER(close_button),close_image);
	gtk_widget_set_size_request(close_button,16,16);*/

	GtkWidget *table = gtk_table_new (3,3,FALSE);
	gtk_table_attach (GTK_TABLE(table),icon,0,1,0,1,GTK_SHRINK,GTK_SHRINK,3,0);
	gtk_table_attach (GTK_TABLE(table),label,1,2,0,1,GTK_SHRINK,GTK_SHRINK,3,1);
	gtk_table_attach (GTK_TABLE(table),empty,2,3,0,1,GTK_EXPAND,GTK_SHRINK,3,0);
	gtk_table_attach (GTK_TABLE(table),image,0,3,1,2,GTK_EXPAND,GTK_SHRINK,3,0);
	gtk_table_attach (GTK_TABLE(table),empty2,0,3,2,3,GTK_SHRINK,GTK_SHRINK,3,1);

	GtkWidget *event_box = gtk_event_box_new ();

	//GtkStyle *style = gtk_widget_get_style (task->tasklist->box);

	GdkColor colorbg = task->tasklist->preview_win_border_color;
	GdkColor colorfg = task->tasklist->preview_win_title_color;

	gtk_widget_modify_bg (event_box,GTK_STATE_NORMAL,&colorbg);
	gtk_widget_modify_fg (label,GTK_STATE_NORMAL,&colorfg);

	gtk_widget_set_tooltip_text (event_box, text);
	
	gtk_container_add (GTK_CONTAINER (event_box), table);

	g_signal_connect_after (event_box, "enter-notify-event",
	                        G_CALLBACK
	                        (talika_tooltips_popup_enter_notify),
	                        task->tasklist);

	g_signal_connect_after (event_box, "leave-notify-event",
	                        G_CALLBACK (talika_tooltips_leave_notify),
	                        task->tasklist);

	g_signal_connect (G_OBJECT (event_box),
	                  "button_press_event",
	                  G_CALLBACK (talika_preview_button_press_event), task);

	return event_box;
}

static GtkWidget*
talika_preview_widget_for_task (TalikaTask *task)
{
	GtkWidget *frame = gtk_frame_new (NULL);
	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_OUT);
	GtkWidget *box = gtk_vbox_new (FALSE, 5);

	GtkWidget *previews;

	if(task->type == TALIKA_TASK_WINDOW)
	{
		if(task->tasklist->previews_group_only)	
		{
			gchar *text = talika_task_get_text (task);
			previews = gtk_label_new (text);
			gtk_widget_modify_fg (previews,GTK_STATE_NORMAL,&task->tasklist->preview_popup_fg_color);
			_make_gtk_label_bold (GTK_LABEL (previews));
			//gtk_label_set_width_chars(GTK_LABEL(child),50);
			//gtk_label_set_line_wrap(GTK_LABEL(child),TRUE);
			gtk_label_set_max_width_chars (GTK_LABEL (previews), 64);
			gtk_label_set_ellipsize (GTK_LABEL (previews),
			                         PANGO_ELLIPSIZE_END);
			g_free (text);
		}
		else
		{
		previews=talika_preview_widget_for_window (task->window, 
		                                           task);
		}
	}
	else if(task->type == TALIKA_TASK_CLASS_GROUP)
	{
		gint row,col;
		row = task->tasklist->preview_rows;
		col = task->tasklist->preview_cols;
		
		previews=gtk_table_new (row,col,FALSE);

		GList *l;
		l = task->windows;

		if(task->tasklist->preview_reverse_order)
			l = g_list_last (l); 

		gint i,j;
		for(i=0; i<row && l; i++)
		{
			for(j=0; j<col && l; j++)
			{
				TalikaTask *class_group_task = l->data;
				GtkWidget *preview = 
					talika_preview_widget_for_window (class_group_task->window, 
					                                  class_group_task);
				gtk_table_attach (GTK_TABLE(previews),preview,
				                  j,j+1,i,i+1,
				                  GTK_SHRINK,GTK_SHRINK,4,4);

				if(task->tasklist->preview_reverse_order)
					l = l->prev; 
				else
					l = l->next;
			}
		}

		//add class info label
		gchar *label_text = talika_task_get_text(task);
		GtkWidget *class_label = gtk_label_new (label_text);
		gtk_widget_modify_fg (class_label,GTK_STATE_NORMAL,&task->tasklist->preview_popup_fg_color);
		g_free (label_text);
		_make_gtk_label_bold (GTK_LABEL (class_label));
		gtk_box_pack_start_defaults (GTK_BOX(box),class_label);
	}
	else if(task->type == TALIKA_TASK_LAUNCHER)
	{
		gchar *text = talika_task_get_text (task);
		previews = gtk_label_new (text);
		gtk_widget_modify_fg (previews,GTK_STATE_NORMAL,&task->tasklist->preview_popup_fg_color);
		_make_gtk_label_bold (GTK_LABEL (previews));
		//gtk_label_set_width_chars(GTK_LABEL(child),50);
		//gtk_label_set_line_wrap(GTK_LABEL(child),TRUE);
		gtk_label_set_max_width_chars (GTK_LABEL (previews), 64);
		gtk_label_set_ellipsize (GTK_LABEL (previews),
		                         PANGO_ELLIPSIZE_END);
		g_free (text);
	}
	else
	{
		//never reah
	}

	gtk_container_add (GTK_CONTAINER (box), previews);
	gtk_container_add (GTK_CONTAINER (frame), box);
	gtk_container_set_border_width (GTK_CONTAINER (box), 10);

	return frame;
}

/*############################ Tooltip popup #################################*/

static void
talika_tooltips_get_popup_xy (TalikaTask * task, 
                              gint * x, gint * y,
                              gint yPadding)
{
	GtkWidget *widget = task->tasklist->popup;
	gint panel_size;
	GtkRequisition requisition;
	gtk_widget_size_request (widget, &requisition);

	gdk_window_get_origin (task->button->window, x, y);

	*x += task->button->allocation.x;
	*y += task->button->allocation.y;

	panel_size = task->tasklist->panel_size;

	if (task->tasklist->panel_orient == PANEL_APPLET_ORIENT_UP)
	{
		*y = *y - requisition.height - yPadding;
	}
	else if (task->tasklist->panel_orient == PANEL_APPLET_ORIENT_DOWN)
	{
		*y = *y + panel_size + yPadding;
	}
	else if (task->tasklist->panel_orient == PANEL_APPLET_ORIENT_RIGHT)
	{
		*x = *x + panel_size + yPadding;
	}
	else
	{
		*x = *x - requisition.width - yPadding;
	}
}

static void
talika_tooltips_position_popup (TalikaTask * task)
{
	gint x, y;
	GtkRequisition event_box_req;
	GtkRequisition popup_req;

	gtk_widget_size_request (task->button, &event_box_req);
	gtk_widget_size_request (task->tasklist->popup, &popup_req);

	talika_tooltips_get_popup_xy (task, &x, &y, 5);

	GdkScreen *screen = gtk_widget_get_screen (task->button);
	GdkRectangle monitor_rect;
	gdk_screen_get_monitor_geometry (screen,
	                                 gdk_screen_get_monitor_at_point
	                                 (screen, x, y), &monitor_rect);
	gint monitor_right = monitor_rect.x + monitor_rect.width;
	gint monitor_left = monitor_rect.x;
	gint monitor_bottom = monitor_rect.y + monitor_rect.height;
	gint monitor_top = monitor_rect.y;


	if (task->tasklist->panel_orient == PANEL_APPLET_ORIENT_UP ||
	    task->tasklist->panel_orient == PANEL_APPLET_ORIENT_DOWN)
	{
		x = x - (popup_req.width / 2) + (event_box_req.width / 2);

		if (x + popup_req.width >= monitor_right - 5)
		{
			x = monitor_right - popup_req.width - 5;
		}
		else if (x < monitor_left + 5)
		{
			x = monitor_left + 5;
		}
	}
	else
	{
		y = y - (popup_req.height / 2) + (event_box_req.height / 2);

		if (y + popup_req.height >= monitor_bottom - 5)
		{
			y = monitor_bottom - popup_req.height - 5;
		}
		else if (y < monitor_top + 5)
		{
			y = monitor_top + 5;
		}
	}

	gtk_window_move (GTK_WINDOW (task->tasklist->popup), x, y);
}

static gboolean
talika_tooltips_plain_popup_expose (GtkWidget * widget, GdkEventExpose * event,
                              gpointer data)
{
	gtk_paint_flat_box (widget->style, widget->window, GTK_STATE_NORMAL,
	                    GTK_SHADOW_IN, &(event->area), widget, "tooltip",
	                    0, 0, widget->allocation.width,
	                    widget->allocation.height);

	return FALSE;
}

static gboolean
talika_tooltips_popup_enter_notify (GtkWidget * widget,
                                    GdkEventCrossing * event,
                                    TalikaTaskList * tasklist)
{
	tasklist->hide_delay_started = FALSE;

	return TRUE;
}

static void
talika_tooltips_attach_task_to_popup (TalikaTask * task,
                                      gboolean preview_enabled)
{
	//remove any existing child from popup
	GtkWidget *last = gtk_bin_get_child (GTK_BIN (task->tasklist->popup));

	if (last)
		gtk_container_remove (GTK_CONTAINER (task->tasklist->popup),
		                      last);
	
	if (preview_enabled)
	{
		GtkWidget *preview = talika_preview_widget_for_task(task);
		gtk_container_add (GTK_CONTAINER (task->tasklist->popup),
		                   preview);
	}
	else
	{
		GtkWidget *box = gtk_vbox_new (FALSE, 5);

		gchar *text = talika_task_get_text (task);
		GtkWidget *child = gtk_label_new (text);
		//_make_gtk_label_bold (GTK_LABEL (child));
		//gtk_label_set_width_chars(GTK_LABEL(child),50);
		//gtk_label_set_line_wrap(GTK_LABEL(child),TRUE);
		gtk_label_set_max_width_chars (GTK_LABEL (child), 64);
		gtk_label_set_ellipsize (GTK_LABEL (child),
		                         PANGO_ELLIPSIZE_END);
		g_free (text);

		gtk_container_add (GTK_CONTAINER (box), child);
		gtk_container_add (GTK_CONTAINER (task->tasklist->popup),
		                   box);
		gtk_container_set_border_width (GTK_CONTAINER (box), 5);
		g_signal_connect (box, "expose_event",
		                  G_CALLBACK (talika_tooltips_plain_popup_expose), 
		                  task);
	}

	gtk_window_resize (GTK_WINDOW (task->tasklist->popup), 10,
		                   10);
}

static GtkWidget *
talika_tooltips_get_popup_plain ()
{
	GtkWidget *popup = gtk_window_new (GTK_WINDOW_POPUP);

	gtk_widget_set_name (popup, "gtk-tooltip");

	return popup;
}

static void draw_preview_mask (GdkDrawable *bitmap, gint width, gint height,TalikaTask * task)
{
	GdkColormap *colormap;
	GdkColor black;
	GdkColor white;
	GdkGC *gc;

	colormap = gdk_colormap_get_system ();
	gdk_color_black (colormap, & black);
	gdk_color_white (colormap, & white);

	gc = gdk_gc_new (bitmap);
	gdk_gc_set_foreground (gc, & black);
	gdk_gc_set_background (gc, & white);

	// fill window_shape_bitmap with black
	gdk_draw_rectangle (bitmap,
	                    gc,
	                    TRUE,  
	                    0,   
	                    0,    
	                    width,
	                    height);

	//draw rounded rectangle
	cairo_t *cr;
	cr = gdk_cairo_create (bitmap);

	double x         = 2;        
	double y         = 2;
	width-=4;
	height-=4;
	
	double aspect        = 1.0;     
	double corner_radius = 8.0;   

	double radius = corner_radius / aspect;
	double degrees = M_PI / 180.0;

	cairo_new_sub_path (cr);
	cairo_arc (cr, x + width - radius,
	           y + radius, radius, -90 * degrees, 0 * degrees);
	cairo_arc (cr, x + width - radius,
	           y + height - radius, radius, 0 * degrees, 90 * degrees);
	cairo_arc (cr, x + radius, 
	           y + height - radius, radius, 90 * degrees, 180 * degrees);
	cairo_arc (cr, x + radius, 
	           y + radius, radius, 180 * degrees, 270 * degrees);
	cairo_close_path (cr);

	cairo_set_source_rgb (cr, (gdouble)task->tasklist->preview_popup_bg_color.red,
					(gdouble)task->tasklist->preview_popup_bg_color.green,
					(gdouble)task->tasklist->preview_popup_bg_color.blue);
	cairo_fill(cr);
	cairo_destroy (cr);
}

static gboolean
talika_tooltips_preview_window_expose (GtkWidget      *widget,
                                       GdkEventExpose *event,TalikaTask * task)
{
	//mask window to a rounded recangular shape
	GdkDrawable *window_shape_bitmap;
	window_shape_bitmap = 
		(GdkDrawable *) gdk_pixmap_new (NULL,
		                                widget->allocation.width,
		                                widget->allocation.height,
		                                1);

	draw_preview_mask (window_shape_bitmap,
	                   widget->allocation.width,
	                   widget->allocation.height,task);

	gtk_widget_shape_combine_mask (widget,
	                               window_shape_bitmap,
	                               0,
	                               0);

	//draw a black border on window
	/*cairo_t *cr;
	cr = gdk_cairo_create (widget->window);

	double x = 0;  
	double y = 0;
	double width  = widget->allocation.width;
	double height = widget->allocation.height;
	double aspect = 1.0;
	double corner_radius = 8.0;

	double radius = corner_radius / aspect;
	double degrees = M_PI / 180.0;

	cairo_new_sub_path (cr);
	cairo_arc (cr, x + width - radius, 
	           y + radius, radius, -90 * degrees, 0 * degrees);
	cairo_arc (cr, x + width - radius, 
	           y + height - radius, radius, 0 * degrees, 90 * degrees);
	cairo_arc (cr, x + radius, 
	           y + height - radius, radius, 90 * degrees, 180 * degrees);
	cairo_arc (cr, x + radius, 
	           y + radius, radius, 180 * degrees, 270 * degrees);
	cairo_close_path (cr);

	cairo_set_source_rgb (cr, (gdouble)task->tasklist->preview_popup_bg_color.red,
					(gdouble)task->tasklist->preview_popup_bg_color.green,
					(gdouble)task->tasklist->preview_popup_bg_color.blue);
	cairo_set_line_width (cr, 2);
	cairo_stroke (cr);
	cairo_destroy (cr);*/

	return FALSE;
}

static gboolean
talika_tooltips_preview_popup_expose (GtkWidget      *widget,
                     GdkEventExpose *event,TalikaTask * task)
{
  cairo_t *cr;
 // cairo_surface_t *cr_surface = gdk_window_ref_cairo_surface(widget->window);

  cr = gdk_cairo_create (widget->window);
  /* the source data is the (composited) event box */
  gdk_cairo_set_source_pixmap (cr, widget->window,
                               widget->allocation.x,
                               widget->allocation.y);
  cairo_set_source_rgba (cr, (gdouble)task->tasklist->preview_popup_bg_color.red/65535,
					(gdouble)task->tasklist->preview_popup_bg_color.green/65535,
					(gdouble)task->tasklist->preview_popup_bg_color.blue/65535,
					(gdouble)(task->tasklist->preview_transparency+1) / 10);
  cairo_set_operator(cr,CAIRO_OPERATOR_SOURCE);
  cairo_paint(cr);
  /* we're done */
  cairo_destroy (cr);
  return FALSE;
}

static GtkWidget *
talika_tooltips_get_popup_preview (TalikaTask * task)
{
	GtkWidget *popup = gtk_window_new (GTK_WINDOW_POPUP);
	
	//gtk_window_set_opacity(GTK_WINDOW(popup), 
	  //                     (gdouble)(task->tasklist->preview_transparency+1) / 10);
	  
	GdkScreen *screen;
	GdkColormap *rgba;
	screen = gtk_widget_get_screen (popup);
	rgba = gdk_screen_get_rgba_colormap (screen);
	gtk_widget_set_colormap (popup, rgba);
	
  
	gtk_widget_set_app_paintable (GTK_WIDGET (popup), TRUE);

	g_signal_connect_after (popup, "enter-notify-event",
	                        G_CALLBACK
	                        (talika_tooltips_popup_enter_notify),
	                        task->tasklist);

	g_signal_connect_after (popup, "leave-notify-event",
	                        G_CALLBACK (talika_tooltips_leave_notify),
	                        task->tasklist);

	g_signal_connect (popup, "expose-event",
		                        G_CALLBACK (talika_tooltips_preview_popup_expose),
		                        task);
		                        
	if(task->tasklist->preview_rounded_corners)
		g_signal_connect_after (popup, "expose-event",
		                        G_CALLBACK (talika_tooltips_preview_window_expose),
		                        task);

	return popup;
}

static void
talika_tooltips_show_popup (TalikaTask * task)
{
	gboolean preview_enabled = task->tasklist->enable_previews &&
		gdk_screen_is_composited (gtk_widget_get_screen(task->button));
	

	//preview_enabled = FALSE;

	if (task->tasklist->popup == NULL)
	{
		if (preview_enabled)
		{
			task->tasklist->popup =
				talika_tooltips_get_popup_preview
				(task);
		}
		else
		{
			task->tasklist->popup =
				talika_tooltips_get_popup_plain ();
		}
	}
		
	talika_tooltips_attach_task_to_popup (task, preview_enabled);

	gtk_widget_show_all (task->tasklist->popup);
	talika_tooltips_position_popup (task);
}


void 
talika_tooltips_hide_popup_immediatly (TalikaTask * task)
{
	if (task->tasklist->popup == NULL)
		return;

	if (task->tasklist->leave_timeout_id)
	{
		g_source_remove (task->tasklist->leave_timeout_id);
		task->tasklist->leave_timeout_id = 0;
	}

	talika_tooltips_hide_popup (task->tasklist);

	talika_tooltips_leave_notify_timeout (task->tasklist);
}

void 
talika_tooltips_free_popup (TalikaTaskList * tasklist)
{
	gtk_widget_destroy (tasklist->popup);
	tasklist->popup = NULL;
}

static void
talika_tooltips_hide_popup (TalikaTaskList * tasklist)
{
	if (tasklist->popup == NULL)
		return;

	gtk_widget_hide_all (tasklist->popup);
	talika_tooltips_free_popup (tasklist);
}

/*############################# Mouse Entered ################################*/

static gboolean
talika_tooltips_enter_notify_timeout (gpointer data)
{
	TalikaTask *task = data;
	task->tasklist->enter_timeout_id = 0;

	if (task->tasklist->cursor_over_icon)
	{
		if (task->tasklist->leave_timeout_id)
		{
			g_source_remove (task->tasklist->leave_timeout_id);
			task->tasklist->leave_timeout_id = 0;
		}

		talika_tooltips_show_popup (task);
	}

	return FALSE;
}


gboolean 
talika_tooltips_enter_notify (GtkWidget * widget,
                              GdkEventCrossing * event, TalikaTask * task)
{
	task->tasklist->hide_delay_started = FALSE;
	task->tasklist->cursor_over_icon = TRUE;

	if (task->tasklist->enter_timeout_id)
		g_source_remove (task->tasklist->enter_timeout_id);

	if (task->tasklist->popup)
	{
		//talika_tooltips_enter_notify_timeout (task);
		task->tasklist->enter_timeout_id =
		g_timeout_add (150, &talika_tooltips_enter_notify_timeout,
		               task);
	}
	else
	{
		task->tasklist->enter_timeout_id =
		g_timeout_add (500, &talika_tooltips_enter_notify_timeout,
		               task);
	}

	return TRUE;
}

void 
talika_tooltips_show_popup_immediatly (TalikaTask * task)
{
	if (task->tasklist->enter_timeout_id)
	{
		g_source_remove (task->tasklist->enter_timeout_id);
		task->tasklist->enter_timeout_id = 0;
	}
		
	talika_tooltips_enter_notify_timeout(task);
}

/*############################# Mouse Leaving ################################*/

static gboolean
talika_tooltips_leave_notify_timeout (gpointer data)
{
	TalikaTaskList *tasklist = data;
	tasklist->leave_timeout_id = 0;

	if (tasklist->hide_delay_started)
	{
		tasklist->hide_delay_started = FALSE;

		if (!tasklist->cursor_over_icon)
			talika_tooltips_hide_popup (tasklist);
	}


	return FALSE;
}

gboolean 
talika_tooltips_leave_notify (GtkWidget * widget,
                              GdkEventCrossing * event,
                              TalikaTaskList * tasklist)
{
	tasklist->cursor_over_icon = FALSE;

	if (!tasklist->hide_delay_started)
	{
		tasklist->hide_delay_started = TRUE;

		if (tasklist->leave_timeout_id)
			g_source_remove (tasklist->leave_timeout_id);

		tasklist->leave_timeout_id =
			g_timeout_add (tasklist->preview_timeout,
			               &talika_tooltips_leave_notify_timeout,
			               tasklist);
	}

	return TRUE;
}

